@@ -2,7 +2,7 @@
  *
  *  Bluetooth packet analyzer - CAPI parser
  *
- *  Copyright (C) 2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *
- *  $Id: capi.c,v 1.5 2004/06/16 09:57:57 holtmann Exp $
+ *  $Id: capi.c,v 1.7 2005/02/01 14:36:59 holtmann Exp $
  */
 
 #include <stdio.h>
@@ -38,8 +38,8 @@
 #include "parser.h"
 
 #define CAPI_U8(frm)  (get_u8(frm))
-#define CAPI_U16(frm) (htobs(htons(get_u16(frm))))
-#define CAPI_U32(frm) (htobl(htonl(get_u32(frm))))
+#define CAPI_U16(frm) (btohs(htons(get_u16(frm))))
+#define CAPI_U32(frm) (btohl(htonl(get_u32(frm))))
 
 static char *cmd2str(uint8_t cmd)
 {
