@@ -3,7 +3,7 @@
  *  Bluetooth packet analyzer - L2CAP parser
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *
- *  $Id: l2cap.c,v 1.34 2004/09/13 19:02:56 holtmann Exp $
+ *  $Id: l2cap.c,v 1.41 2005/02/01 18:51:42 holtmann Exp $
  */
 
 #include <stdio.h>
@@ -52,6 +52,7 @@ static handle_info handle_table[HANDLE_TABLE_SIZE];
 typedef struct {
 	uint16_t cid;
 	uint16_t psm;
+	uint16_t num;
 } cid_info;
 #define CID_TABLE_SIZE 20
 
@@ -88,14 +89,21 @@ static struct frame *get_frame(uint16_t handle)
 static void add_cid(int in, uint16_t cid, uint16_t psm)
 {
 	register cid_info *table = cid_table[in];
-	register int i;
+	register int i, pos = -1;
+	uint16_t num = 1;
 
-	for (i = 0; i < CID_TABLE_SIZE; i++)
-		if (!table[i].cid || table[i].cid == cid) {
-			table[i].cid = cid;
-			table[i].psm = psm;
-			break;
-		}
+	for (i = 0; i < CID_TABLE_SIZE; i++) {
+		if ((pos < 0 && !table[i].cid) || table[i].cid == cid)
+			pos = i;
+		if (table[i].psm == psm)
+			num++;
+	}
+
+	if (pos >= 0) {
+		table[pos].cid = cid;
+		table[pos].psm = psm;
+		table[pos].num = num;
+	}
 }
 
 static void del_cid(int in, uint16_t dcid, uint16_t scid)
@@ -115,6 +123,8 @@ static void del_cid(int in, uint16_t dcid, uint16_t scid)
 		for (i = 0; i < CID_TABLE_SIZE; i++)
 			if (cid_table[t][i].cid == cid[t]) {
 				cid_table[t][i].cid = 0;
+				cid_table[t][i].psm = 0;
+				cid_table[t][i].num = 0;
 				break;
 			}
 	}
@@ -131,6 +141,17 @@ static uint16_t get_psm(int in, uint16_t cid)
 	return parser.defpsm;
 }
 
+static uint16_t get_num(int in, uint16_t cid)
+{
+	register cid_info *table = cid_table[in];
+	register int i;
+
+	for (i = 0; i < CID_TABLE_SIZE; i++)
+		if (table[i].cid == cid)
+			return table[i].num;
+	return 0;
+}
+
 static uint32_t get_val(uint8_t *ptr, uint8_t len)
 {
 	switch (len) {
@@ -144,6 +165,93 @@ static uint32_t get_val(uint8_t *ptr, uint8_t len)
 	return 0;
 }
 
+static char *reason2str(uint16_t reason)
+{
+	switch (reason) {
+	case 0x0000:
+		return "Command not understood";
+	case 0x0001:
+		return "Signalling MTU exceeded";
+	case 0x0002:
+		return "Invalid CID in request";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *connresult2str(uint16_t result)
+{
+	switch (result) {
+	case 0x0000:
+		return "Connection successful";
+	case 0x0001:
+		return "Connection pending";
+	case 0x0002:
+		return "Connection refused - PSM not supported";
+	case 0x0003:
+		return "Connection refused - security block";
+	case 0x0004:
+		return "Connection refused - no resources available";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *status2str(uint16_t status)
+{
+	switch (status) {
+	case 0x0000:
+		return "No futher information available";
+	case 0x0001:
+		return "Authentication pending";
+	case 0x0002:
+		return "Authorization pending";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *confresult2str(uint16_t result)
+{
+	switch (result) {
+	case 0x0000:
+		return "Success";
+	case 0x0001:
+		return "Failure - unacceptable parameters";
+	case 0x0002:
+		return "Failure - rejected (no reason provided)";
+	case 0x0003:
+		return "Failure - unknown options";
+	default:
+		return "Reserved";
+	}
+}
+static char *inforesult2str(uint16_t result)
+{
+	switch (result) {
+	case 0x0000:
+		return "Success";
+	case 0x0001:
+		return "Not supported";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *type2str(uint8_t type)
+{
+	switch (type) {
+	case 0x00:
+		return "No traffic";
+	case 0x01:
+		return "Best effort";
+	case 0x02:
+		return "Guaranteed";
+	default:
+		return "Reserved";
+	}
+}
+
 static char *mode2str(uint8_t mode)
 {
 	switch (mode) {
@@ -161,37 +269,82 @@ static char *mode2str(uint8_t mode)
 static inline void command_rej(int level, struct frame *frm)
 {
 	l2cap_cmd_rej *h = frm->ptr;
+	uint16_t reason = btohs(h->reason);
+	uint32_t cid;
 
-	printf("Command rej: reason %d\n", btohs(h->reason));
+	printf("Command rej: reason %d", reason);
+
+	switch (reason) {
+	case 0x0001:
+		printf(" mtu %d\n", get_val(frm->ptr + L2CAP_CMD_REJ_SIZE, 2));
+		break;
+	case 0x0002:
+		cid = get_val(frm->ptr + L2CAP_CMD_REJ_SIZE, 4);
+		printf(" dcid 0x%4.4x scid 0x%4.4x\n", cid & 0xffff, cid >> 16);
+		break;
+	default:
+		printf("\n");
+		break;
+	}
+
+	if (parser.flags & DUMP_VERBOSE) {
+		p_indent(level + 1, frm);
+		printf("%s\n", reason2str(reason));
+	}
 }
 
 static inline void conn_req(int level, struct frame *frm)
 {
 	l2cap_conn_req *h = frm->ptr;
+	uint16_t psm = btohs(h->psm);
+	uint16_t scid = btohs(h->scid);
 
-	add_cid(frm->in, btohs(h->scid), btohs(h->psm));
+	add_cid(frm->in, scid, psm);
 
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Connect req: psm %d scid 0x%4.4x\n",
-				btohs(h->psm), btohs(h->scid));
+	printf("Connect req: psm %d scid 0x%4.4x\n", psm, scid);
 }
 
 static inline void conn_rsp(int level, struct frame *frm)
 {
 	l2cap_conn_rsp *h = frm->ptr;
+	uint16_t scid = btohs(h->scid);
+	uint16_t dcid = btohs(h->dcid);
+	uint16_t result = btohs(h->result);
+	uint16_t status = btohs(h->status);
 	uint16_t psm;
 
-	if ((psm = get_psm(!frm->in, btohs(h->scid))))
-		add_cid(frm->in, btohs(h->dcid), psm);
+	switch (h->result) {
+	case L2CAP_CR_SUCCESS:
+		if ((psm = get_psm(!frm->in, scid)))
+			add_cid(frm->in, dcid, psm);
+		break;
+
+	case L2CAP_CR_PEND:
+		break;
+
+	default:
+		del_cid(frm->in, dcid, scid);
+		break;
+	}
 
 	if (p_filter(FILT_L2CAP))
 		return;
 
 	printf("Connect rsp: dcid 0x%4.4x scid 0x%4.4x result %d status %d\n",
-				btohs(h->dcid), btohs(h->scid),
-				btohs(h->result), btohs(h->status));
+		dcid, scid, result, status);
+
+	if (parser.flags & DUMP_VERBOSE) {
+		p_indent(level + 1, frm);
+		printf("%s", connresult2str(result));
+
+		if (result == 0x0001)
+			printf(" - %s\n", status2str(status));
+		else
+			printf("\n");
+	}
 }
 
 static void conf_opt(int level, void *ptr, int len)
@@ -203,26 +356,43 @@ static void conf_opt(int level, void *ptr, int len)
 		ptr += L2CAP_CONF_OPT_SIZE + h->len;
 		len -= L2CAP_CONF_OPT_SIZE + h->len;
 
-		switch (h->type) {
+		if (h->type & 0x80)
+			printf("[");
+
+		switch (h->type & 0x7f) {
 		case L2CAP_CONF_MTU:
-			printf("MTU ");
+			printf("MTU");
 			if (h->len > 0)
-				printf("%d ", get_val(h->val, h->len));
+				printf(" %d", get_val(h->val, h->len));
 			break;
+
 		case L2CAP_CONF_FLUSH_TO:
-			printf("FlushTO ");
+			printf("FlushTO");
 			if (h->len > 0)
-				printf("%d ", get_val(h->val, h->len));
+				printf(" %d", get_val(h->val, h->len));
 			break;
-		case L2CAP_CONF_RFC_MODE:
-			printf("Mode ");
+
+		case L2CAP_CONF_QOS:
+			printf("QoS");
 			if (h->len > 0)
-				printf("%d (%s) ", *h->val, mode2str(*h->val));
+				printf(" 0x%02x (%s)", *(h->val + 1), type2str(*(h->val + 1)));
 			break;
+
+		case L2CAP_CONF_RFC:
+			printf("Mode");
+			if (h->len > 0)
+				printf(" 0x02%x (%s)", *h->val, mode2str(*h->val));
+			break;
+
 		default:
-			printf("Unknown (type %2.2x, len %d) ", h->type, h->len);
+			printf("Unknown (type %2.2x, len %d)", h->type & 0x7f, h->len);
 			break;
 		}
+
+		if (h->type & 0x80)
+			printf("] ");
+		else
+			printf(" ");
 	}
 	printf("\n");
 }
@@ -235,24 +405,32 @@ static inline void conf_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Config req: dcid 0x%4.4x flags 0x%4.4x clen %d\n",
+	printf("Config req: dcid 0x%4.4x flags 0x%2.2x clen %d\n",
 			btohs(h->dcid), btohs(h->flags), clen);
+
 	if (clen)
-		conf_opt(level, h->data, clen);
+		conf_opt(level + 1, h->data, clen);
 }
 
 static inline void conf_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 {
 	l2cap_conf_rsp *h = frm->ptr;
+	uint16_t result = btohs(h->result);
 	int clen = btohs(cmd->len) - L2CAP_CONF_RSP_SIZE;
 
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Config rsp: scid 0x%4.4x flags 0x%4.4x result %d clen %d\n",
-			btohs(h->scid), btohs(h->flags), btohs(h->result), clen);
+	printf("Config rsp: scid 0x%4.4x flags 0x%2.2x result %d clen %d\n",
+			btohs(h->scid), btohs(h->flags), result, clen);
+
+	if (parser.flags & DUMP_VERBOSE) {
+		p_indent(level + 1, frm);
+		printf("%s\n", confresult2str(result));
+	}
+
 	if (clen)
-		conf_opt(level, h->data, clen);
+		conf_opt(level + 1, h->data, clen);
 }
 
 static inline void disconn_req(int level, struct frame *frm)
@@ -269,7 +447,10 @@ static inline void disconn_req(int level, struct frame *frm)
 static inline void disconn_rsp(int level, struct frame *frm)
 {
 	l2cap_disconn_rsp *h = frm->ptr;
-	del_cid(frm->in, btohs(h->dcid), btohs(h->scid));
+	uint16_t dcid = btohs(h->dcid);
+	uint16_t scid = btohs(h->scid);
+
+	del_cid(frm->in, dcid, scid);
 
 	if (p_filter(FILT_L2CAP))
 		return;
@@ -299,6 +480,7 @@ static inline void echo_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 static void info_opt(int level, int type, void *ptr, int len)
 {
 	p_indent(level, 0);
+
 	switch (type) {
 	case 0x0001:
 		printf("Connectionless MTU %d\n", get_val(ptr, len));
@@ -325,14 +507,22 @@ static inline void info_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 static inline void info_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 {
 	l2cap_info_rsp *h = frm->ptr;
+	uint16_t type = btohs(h->type);
+	uint16_t result = btohs(h->result);
 	int ilen = btohs(cmd->len) - L2CAP_INFO_RSP_SIZE;
 
 	if (p_filter(FILT_L2CAP))
 		return;
 
-	printf("Info rsp: type %d result %d\n", btohs(h->type), btohs(h->result));
+	printf("Info rsp: type %d result %d\n", type, result);
+
+	if (parser.flags & DUMP_VERBOSE) {
+		p_indent(level + 1, frm);
+		printf("%s\n", inforesult2str(result));
+	}
+
 	if (ilen)
-		info_opt(level, btohs(h->type), h->data, ilen);
+		info_opt(level + 1, type, h->data, ilen);
 }
 
 static void l2cap_parse(int level, struct frame *frm)
@@ -433,6 +623,7 @@ static void l2cap_parse(int level, struct frame *frm)
 		uint32_t proto;
 
 		frm->cid = cid;
+		frm->num = get_num(!frm->in, cid);
 
 		if (!p_filter(FILT_L2CAP)) {
 			p_indent(level, frm);
@@ -544,6 +735,7 @@ void l2cap_dump(int level, struct frame *frm)
 		fr->ts  = frm->ts;
 		fr->handle  = frm->handle;
 		fr->cid     = frm->cid;
+		fr->num     = frm->num;
 		fr->channel = frm->channel;
 	} else {
 		if (!(fr = get_frame(frm->handle))) {
