@@ -3,7 +3,7 @@
  *  Bluetooth packet analyzer - Common parser functions
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *
- *  $Id: parser.h,v 1.33 2004/09/13 19:09:44 holtmann Exp $
+ *  $Id: parser.h,v 1.39 2005/02/01 14:36:59 holtmann Exp $
  */
 
 #ifndef __PARSER_H
@@ -29,6 +29,8 @@
 
 #include <bluetooth/bluetooth.h>
 
+#include <netinet/in.h>
+
 struct frame {
 	void	*data;
 	int	data_len;
@@ -37,7 +39,8 @@ struct frame {
 	int	in;
 	int	handle;
 	int	cid;
-	int 	channel;
+	int	num;
+	int	channel;
 	long	flags;
 	struct	timeval ts;
 };
@@ -49,7 +52,9 @@ struct frame {
 #define DUMP_HEX	0x02
 #define DUMP_EXT	0x04
 #define DUMP_RAW	0x08
-#define DUMP_TSTAMP	0x10
+#define DUMP_BPA	0x10
+#define DUMP_VERBOSE	0x40
+#define DUMP_TSTAMP	0x80
 #define DUMP_TYPE_MASK	(DUMP_ASCII | DUMP_HEX | DUMP_EXT)
 
 /* Parser filter */
@@ -66,7 +71,8 @@ struct frame {
 
 #define FILT_OBEX	0x00010000
 #define FILT_CAPI	0x00020000
-#define FILT_CSR	0x1000000f
+#define FILT_CSR	0x1000000a
+#define FILT_BPA	0x1000000f
 
 #define STRUCT_OFFSET(type, member)  ((uint8_t *)&(((type *)NULL)->member) - \
                                      (uint8_t *)((type *)NULL))
@@ -180,6 +186,7 @@ void avdtp_dump(int level, struct frame *frm);
 void obex_dump(int level, struct frame *frm);
 void capi_dump(int level, struct frame *frm);
 void csr_dump(int level, struct frame *frm);
+void bpa_dump(int level, struct frame *frm);
 
 static inline void parse(struct frame *frm)
 {
