@@ -3,7 +3,7 @@
  *  Bluetooth packet analyzer - SDP parser
  *
  *  Copyright (C) 2001-2002  Ricky Yuen <ryuen@qualcomm.com>
- *  Copyright (C) 2003-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *
- *  $Id: sdp.h,v 1.11 2004/06/09 13:42:58 holtmann Exp $
+ *  $Id: sdp.h,v 1.14 2005/02/01 14:37:00 holtmann Exp $
  */
 
 #ifndef __SDP_H
@@ -75,6 +75,8 @@
 #define SDP_UUID_INTERCOM                              0x1110
 #define SDP_UUID_FAX                                   0x1111
 #define SDP_UUID_HEADSET_AUDIO_GATEWAY                 0x1112
+#define SDP_UUID_WAP                                   0x1113
+#define SDP_UUID_WAP_CLIENT                            0x1114
 #define SDP_UUID_PANU                                  0x1115 /* PAN */
 #define SDP_UUID_NAP                                   0x1116 /* PAN */
 #define SDP_UUID_GN                                    0x1117 /* PAN */
@@ -82,11 +84,14 @@
 #define SDP_UUID_IMAGING_RESPONDER                     0x111b /* BIP */
 #define SDP_UUID_IMAGING_AUTOMATIC_ARCHIVE             0x111c /* BIP */
 #define SDP_UUID_IMAGING_REFERENCED_OBJECTS            0x111d /* BIP */
+#define SDP_UUID_HANDSFREE                             0x111e
+#define SDP_UUID_HANDSFREE_AUDIO_GATEWAY               0x111f
 #define SDP_UUID_HUMAN_INTERFACE_DEVICE                0x1124 /* HID */
 #define SDP_UUID_HARDCOPY_CABLE_REPLACE                0x1125 /* HCRP */
 #define SDP_UUID_HCR_PRINT                             0x1126 /* HCRP */
 #define SDP_UUID_HCR_SCAN                              0x1127 /* HCRP */
 #define SDP_UUID_COMMON_ISDN_ACCESS                    0x1128 /* CIP */
+#define SDP_UUID_SIM_ACCESS                            0x112d /* SAP */
 #define SDP_UUID_PNP_INFORMATION                       0x1200
 #define SDP_UUID_GENERIC_NETWORKING                    0x1201
 #define SDP_UUID_GENERIC_FILE_TRANSFER                 0x1202
